@@ -65,12 +65,7 @@ static struct attribute *bcma_device_attrs[] = {
 	&dev_attr_class.attr,
 	NULL,
 };
-#if LINUX_VERSION_IS_GEQ(3,12,0)
 ATTRIBUTE_GROUPS(bcma_device);
-#else
-#define BP_ATTR_GRP_STRUCT device_attribute
-ATTRIBUTE_GROUPS_BACKPORT(bcma_device);
-#endif
 
 static struct bus_type bcma_bus_type = {
 	.name		= "bcma",
@@ -78,12 +73,7 @@ static struct bus_type bcma_bus_type = {
 	.probe		= bcma_device_probe,
 	.remove		= bcma_device_remove,
 	.uevent		= bcma_device_uevent,
-#if LINUX_VERSION_IS_GEQ(3,12,0)
 	.dev_groups	= bcma_device_groups,
-#else
-	.dev_attrs = bcma_device_dev_attrs,
-#endif
-
 };
 
 static u16 bcma_cc_core_id(struct bcma_bus *bus)
@@ -166,7 +156,6 @@ static struct device_node *bcma_of_find_child_device(struct device *parent,
 	return NULL;
 }
 
-#if LINUX_VERSION_IS_GEQ(3,13,0)
 static int bcma_of_irq_parse(struct device *parent,
 			     struct bcma_device *core,
 			     struct of_phandle_args *out_irq, int num)
@@ -206,13 +195,6 @@ static unsigned int bcma_of_get_irq(struct device *parent,
 
 	return irq_create_of_mapping(&out_irq);
 }
-#else
-static unsigned int bcma_of_get_irq(struct device *parent,
-				    struct bcma_device *core, int num)
-{
-	return 0;
-}
-#endif
 
 static void bcma_of_fill_device(struct device *parent,
 				struct bcma_device *core)
@@ -254,7 +236,6 @@ EXPORT_SYMBOL(bcma_core_irq);
 
 void bcma_prepare_core(struct bcma_bus *bus, struct bcma_device *core)
 {
-	device_initialize(&core->dev);
 	core->dev.release = bcma_release_core_dev;
 	core->dev.bus = &bcma_bus_type;
 	dev_set_name(&core->dev, "bcma%d:%d", bus->num, core->core_index);
@@ -296,10 +277,11 @@ static void bcma_register_core(struct bcma_bus *bus, struct bcma_device *core)
 {
 	int err;
 
-	err = device_add(&core->dev);
+	err = device_register(&core->dev);
 	if (err) {
 		bcma_err(bus, "Could not register dev for core 0x%03X\n",
 			 core->id.id);
+		put_device(&core->dev);
 		return;
 	}
 	core->dev_registered = true;
@@ -390,7 +372,7 @@ void bcma_unregister_cores(struct bcma_bus *bus)
 	/* Now noone uses internally-handled cores, we can free them */
 	list_for_each_entry_safe(core, tmp, &bus->cores, list) {
 		list_del(&core->list);
-		put_device(&core->dev);
+		kfree(core);
 	}
 }
 
@@ -669,7 +651,6 @@ static int __init bcma_init_bus_register(void)
 	if (bcma_bus_registered)
 		return 0;
 
-	init_bcma_device_attrs();
 	err = bus_register(&bcma_bus_type);
 	if (!err)
 		bcma_bus_registered = 1;
